<?php
/**
 * PoiController class
 *
 * PHP version 5
 *
 * @category ADPOI
 * @package  Api.controller
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * PoiController is the controller class
 * for managing Poi records
 *
 * @category ADPOI
 * @package  Api.controller
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class PoiController extends RestApiController
{
    /**
     * Returns controller's actions' filters
     * 
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'validInput',
            'accessControl',
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * 
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow',
                'roles' => array(AuthManager::OPERATION_POI_CREATE),
                'actions' => array('create'),
            ),
            array('allow',
                'roles' => array(
                    array(
                        AuthManager::OPERATION_POI_UPDATE,
                        'poi_id' => Yii::app()->request->getParam('id'),
                    ),
                ),
                'actions' => array('update'),
            ),
            array('allow',
                'roles' => array(
                    array(
                        AuthManager::OPERATION_POI_DELETE,
                        'poi_id' => Yii::app()->request->getParam('id'),
                    ),
                ),
                'actions' => array('delete'),
            ),
            array('allow',
                'roles' => array(AuthManager::OPERATION_POI_SHOW),
                'actions' => array('show'),
            ),
            array('allow',
                'roles' => array(AuthManager::OPERATION_POI_SEARCH),
                'actions' => array('list'),
            ),
            array('deny',
                'users' => array('*'),
            ),
        );
    }
    
    /**
     * Returns api methods' models configuration
     * 
     * @return api models configuration
     * @see RestApiController::models()
     */
    public function models()
    {
        $user = Yii::app()->user;
        return array(
            'show' => array(
                'class' => 'PoiApiModel',
                'scenario' => 'show',
                'criteria' => array(
                    'with' => array('photosCount', 'votesCount'),
                ),
            ),
            'create' => array(
                'class' => 'PoiApiModel',
                'scenario' => 'create',
                'user_id' => $user->isGuest ? null : $user->logged->id,
            ),
            'update' => array(
                'class' => 'PoiApiModel',
                'scenario' => 'update',
            
            ),
            'delete' => array(
                'class' => 'PoiApiModel',
                'scenario' => 'delete',
            ),
            'list' => array(
                'class' => 'PoiApiModel',
                'scenario' => 'list',
                'user_id' => $user->isGuest ? null : $user->logged->id,
            ),
        );
    }
   
    /**
     * Returns attached actions configuration
     * 
     * @return array attached actions
     * @see CController::actions()
     */
    public function actions()
    {
        return array(
            'create' => array(
                'class' => 'RestApiCreateMethod',
                'responseMap' => array(
                    'id' => 'poi',
                ),
            ),
            'update' => array(
                'class' => 'RestApiUpdateMethod',
            ),
            'show' => array(
                'class' => 'RestApiShowMethod',
                'responseMap' => array(
                    'name' => '#',
                    'address' => '#',
                    'description' => '#',
                    'latitude' => '#',
                    'longitude' => '#',
                    'phone' => '#',
                    'email' => '#',
                    'skype' => '#',
                    'exp:$data->getRating()' => 'rating',
                    'exp:$data->votesCount' => 'opinions',
                    'exp:$data->photosCount' => 'photos',
                    'exp:$data->visitorsCount' => 'visitors',
                ),
            ),
            'delete' => array(
                'class' => 'RestApiDeleteMethod',
            ),
            'list' => array(
                'class' => 'RestApiListMethod',
                'responseMap' => array(
                    'exp:$data->getData()' => array(
                        'items',
                        '*' => array(
                            'id' => '#',
                            'name' => '#',
                            'address' => '#',
                            'latitude' => '#',
                            'longitude' => '#',
                            'distance' => '#',
                        ),
                    ),
                    'exp:$data->getPagination()->getPageCount()' => 'pages',
                ),
            ),
        );
    }
}
